@@ -1,22 +1,27 @@
 #!/bin/bash
 
-OUTPUT=${1:?"output file required"}
+if (( $# < 1 )); then
+    echo "Usage: $0 <input diff files...>"
+    exit 1
+fi
 
 SELF_DIR=$(cd $(dirname $0) && pwd) || exit 1
 YDIFF_PY=$SELF_DIR/../ydiff.py
 
-# To test with py3k: PYTHON=python3 make test
-PYTHON=${PYTHON:-python}
+PYTHON=${PYTHON:-python3}
+unset YDIFF_OPTIONS
 
 set -o errexit
-STATS="stats.$$.tmp"
 
-for i in {1..100}; do cat "tests/svn/in.diff"; done \
+STATS="stats.$$.tmp"
+OUTPUT="profile.$$.tmp"
+
+cat "$@" \
     | $PYTHON -m cProfile -o $STATS $YDIFF_PY -c always -s -w 60 \
     > /dev/null
 
 $PYTHON -c "import pstats;  p = pstats.Stats('$STATS'); \
-    p.strip_dirs().sort_stats('time').print_stats('ydiff')" \
+    p.strip_dirs().sort_stats('time').print_stats('ydiff|difflib')" \
     | tee $OUTPUT
 
 rm -f $STATS
