@@ -13,19 +13,119 @@ sys.path.insert(0, '')
 import ydiff  # nopep8
 
 
+class SplitToWordsTest(unittest.TestCase):
+
+    def test_ok(self):
+        tests = [
+            # (input, want)
+            ('HELLO_WORLD = 0', ['HELLO', '_', 'WORLD', ' ', '=', ' ', '0']),
+            ('class HelloWorld\n', ['class', ' ', 'Hello', 'World', '\n']),
+            ('int foo3\n', ['int', ' ', 'foo', '3', '\n']),
+            ('int f3\n', ['int', ' ', 'f3', '\n']),
+            ('a.addOption', ['a', '.', 'add', 'Option']),
+            ('a.AddOption', ['a', '.', 'Add', 'Option']),
+            ('class HelloW0rld\n', ['class', ' ', 'Hello', 'W0rld', '\n']),
+            ('hello_w0rld++\n', ['hello', '_', 'w0rld', '+', '+', '\n']),
+            ('var foo []string', ['var', ' ', 'foo', ' ', '[', ']', 'string']),
+            ('i -= 1\n', ['i', ' ', '-', '=', ' ', '1', '\n']),
+        ]
+        for s, want in tests:
+            got = ydiff._split_to_words(s)
+            self.assertEqual(want, got)
+
+
+class StrSplitTest(unittest.TestCase):
+
+    def test_not_colorized(self):
+        text = 'Hi, 你好\n'
+        tests = [
+            # (width, want)
+            (4, ('Hi, ', '你好\n', 4)),
+            (5, ('Hi, 你', '好\n', 6)),
+            (8, ('Hi, 你好', '\n', 8)),
+            (9, ('Hi, 你好\n', '', 9)),
+            (10, ('Hi, 你好\n', '', 9)),
+        ]
+        for width, want in tests:
+            got = ydiff._strsplit(text, width, {})
+            self.assertEqual(want, got)
+
+    def test_colorized(self):
+        g = '\x1b[32m'  # green
+        b = '\x1b[34m'  # blue
+        r = '\x1b[0m'   # reset
+        # Width:     1----2----------3-----[4]5--[6]7
+        parts = [g, 'H', 'i', r, b, '!', r, '你', '好']
+        codes = {g, b, r}
+        tests = [
+            # (width, want_left, want_right, want_width)
+            (1, (g, 'H', r), (g, 'i', r, b, '!', r, '你好'), 1),
+            (2, (g, 'Hi', r, b, r), (b, '!', r, '你好'), 2),
+            (3, (g, 'Hi', r, b, '!', r), ('你好'), 3),
+            (4, (g, 'Hi', r, b, '!', r, '你'), ('好',), 5),
+            (5, (g, 'Hi', r, b, '!', r, '你'), ('好',), 5),
+            (6, (g, 'Hi', r, b, '!', r, '你好'), (), 7),
+            (7, (g, 'Hi', r, b, '!', r, '你好'), (), 7),
+            (8, (g, 'Hi', r, b, '!', r, '你好'), (), 7),
+        ]
+        for width, want_left, want_right, want_width in tests:
+            got = ydiff._strsplit(''.join(parts), width, codes)
+            self.assertEqual(''.join(want_left), got[0])
+            self.assertEqual(''.join(want_right), got[1])
+            self.assertEqual(want_width, got[2])
+
+
+class StrTrimTest(unittest.TestCase):
+
+    def test_not_colorized(self):
+        text = 'Hi, 你好\n'
+        tests = [
+            # (width, want)
+            (4, ('Hi, ', '你好\n', 4)),
+            (5, ('Hi, 你', '好\n', 6)),
+            (8, ('Hi, 你好', '\n', 8)),
+            (9, ('Hi, 你好\n', '', 9)),
+            (10, ('Hi, 你好\n', '', 9)),
+        ]
+        for width, want in tests:
+            got = ydiff._strsplit(text, width, {})
+            self.assertEqual(want, got)
+
+    def test_colorized(self):
+        g = '\x1b[32m'  # green
+        b = '\x1b[34m'  # blue
+        r = '\x1b[0m'   # reset
+        # Width:     1----2----------3-----[4]5--[6]7
+        parts = [g, 'H', 'i', r, b, '!', r, '你', '好']
+        codes = {g, b, r}
+        tests = [
+            # (width, pad, want)
+            (1, False, (g, r, '>')),
+            (2, False, (g, 'H', r, '>')),
+            (3, False, (g, 'Hi', r, b, r, '>')),
+            (4, False, (g, 'Hi', r, b, '!', r, '>')),
+            (5, False, (g, 'Hi', r, b, '!', r, '你>')),
+            (6, False, (g, 'Hi', r, b, '!', r, '你>')),
+            (7, False, (g, 'Hi', r, b, '!', r, '你好')),
+            (8, False, (g, 'Hi', r, b, '!', r, '你好')),
+            (8, True, (g, 'Hi', r, b, '!', r, '你好 ')),
+        ]
+        for width, pad, want in tests:
+            got = ydiff._strtrim(''.join(parts), width, '>', pad, codes)
+            self.assertEqual(''.join(want), got, 'width %d failed' % width)
+
+
 class DecodeTest(unittest.TestCase):
 
     def test_normal(self):
-        utext = 'hello'.encode('utf-8')
-        self.assertEqual('hello', ydiff.decode(utext))
+        octets = b'\xe4\xbd\xa0\xe5\xa5\xbd'
+        want = '你好'
+        self.assertEqual(ydiff._decode(octets), want)
 
     def test_latin_1(self):
-        text = '\x80\x02q\x01(U'
-        if sys.version_info[0] == 2:
-            decoded_text = text.decode('latin-1')
-        else:
-            decoded_text = text
-        self.assertEqual(decoded_text, ydiff.decode(text))
+        octets = b'\x80\x02q\x01(U'
+        want = '\x80\x02q\x01(U'
+        self.assertEqual(ydiff._decode(octets), want)
 
 
 class HunkTest(unittest.TestCase):
@@ -54,8 +154,8 @@ class DiffMarkupTest(unittest.TestCase):
             +++ new
             hunk header
             @@ -1,5 +1,5 @@
-            -hhello
-            +helloo
+            -_hello
+            +hello+
             +spammm
              world
             -garb
@@ -67,13 +167,13 @@ class DiffMarkupTest(unittest.TestCase):
 
         hunk = ydiff.Hunk(['hunk header\n'], '@@ -1,5 +1,5 @@\n',
                           (1, 5), (1, 5))
-        hunk.append(('-', 'hhello\n'))
-        hunk.append(('+', 'helloo\n'))
+        hunk.append(('-', '_hello\n'))
+        hunk.append(('+', 'hello+\n'))
         hunk.append(('+', 'spammm\n'))
         hunk.append((' ', 'world\n'))
         hunk.append(('-', 'garb\n'))
         hunk.append(('-', 'Again\n'))
-        hunk.append(('-', '	tabbed\n'))
+        hunk.append(('-', '\ttabbed\n'))
         hunk.append(('+', 'again\n'))
         hunk.append(('+', ' spaced\n'))
         diff = ydiff.UnifiedDiff(
@@ -90,8 +190,8 @@ class DiffMarkupTest(unittest.TestCase):
 
         self.assertEqual(out[0], '\x1b[33m--- old\n\x1b[0m')
         self.assertEqual(out[1], '\x1b[33m+++ new\n\x1b[0m')
-        self.assertEqual(out[2], '\x1b[1;36mhunk header\n\x1b[0m')
-        self.assertEqual(out[3], '\x1b[1;34m@@ -0 +0 @@\n\x1b[0m')
+        self.assertEqual(out[2], '\x1b[36mhunk header\n\x1b[0m')
+        self.assertEqual(out[3], '\x1b[34m@@ -0 +0 @@\n\x1b[0m')
 
     def test_markup_traditional_old_changed(self):
         hunk = ydiff.Hunk([], '@@ -1 +0,0 @@\n', (1, 0), (0, 0))
@@ -104,8 +204,8 @@ class DiffMarkupTest(unittest.TestCase):
 
         self.assertEqual(out[0], '\x1b[33m--- old\n\x1b[0m')
         self.assertEqual(out[1], '\x1b[33m+++ new\n\x1b[0m')
-        self.assertEqual(out[2], '\x1b[1;34m@@ -1 +0,0 @@\n\x1b[0m')
-        self.assertEqual(out[3], '\x1b[1;31m-spam\n\x1b[0m')
+        self.assertEqual(out[2], '\x1b[34m@@ -1 +0,0 @@\n\x1b[0m')
+        self.assertEqual(out[3], '\x1b[31m-spam\n\x1b[0m')
 
     def test_markup_traditional_new_changed(self):
         hunk = ydiff.Hunk([], '@@ -0,0 +1 @@\n', (0, 0), (1, 0))
@@ -118,13 +218,13 @@ class DiffMarkupTest(unittest.TestCase):
 
         self.assertEqual(out[0], '\x1b[33m--- old\n\x1b[0m')
         self.assertEqual(out[1], '\x1b[33m+++ new\n\x1b[0m')
-        self.assertEqual(out[2], '\x1b[1;34m@@ -0,0 +1 @@\n\x1b[0m')
+        self.assertEqual(out[2], '\x1b[34m@@ -0,0 +1 @@\n\x1b[0m')
         self.assertEqual(out[3], '\x1b[32m+spam\n\x1b[0m')
 
     def test_markup_traditional_both_changed(self):
         hunk = ydiff.Hunk([], '@@ -1,2 +1,2 @@\n', (1, 2), (1, 2))
-        hunk.append(('-', 'hella\n'))
-        hunk.append(('+', 'hello\n'))
+        hunk.append(('-', 'hell-\n'))
+        hunk.append(('+', 'hell+\n'))
         hunk.append((' ', 'common\n'))
         diff = ydiff.UnifiedDiff([], '--- old\n', '+++ new\n', [hunk])
         marker = ydiff.DiffMarker()
@@ -134,15 +234,15 @@ class DiffMarkupTest(unittest.TestCase):
 
         self.assertEqual(out[0], '\x1b[33m--- old\n\x1b[0m')
         self.assertEqual(out[1], '\x1b[33m+++ new\n\x1b[0m')
-        self.assertEqual(out[2], '\x1b[1;34m@@ -1,2 +1,2 @@\n\x1b[0m')
+        self.assertEqual(out[2], '\x1b[34m@@ -1,2 +1,2 @@\n\x1b[0m')
         self.assertEqual(
             out[3],
-            '\x1b[1;31m-\x1b[0m\x1b[31mhell'
-            '\x1b[4m\x1b[31ma\x1b[0m\x1b[31m\n\x1b[0m')
+            '\x1b[31m-\x1b[0m\x1b[31mhell'
+            '\x1b[7m\x1b[31m-\x1b[0m\x1b[31m\n\x1b[0m')
         self.assertEqual(
             out[4],
             '\x1b[32m+\x1b[0m\x1b[32mhell'
-            '\x1b[4m\x1b[32mo\x1b[0m\x1b[32m\n\x1b[0m')
+            '\x1b[7m\x1b[32m+\x1b[0m\x1b[32m\n\x1b[0m')
         self.assertEqual(out[5], '\x1b[0m common\n\x1b[0m')
 
     def test_markup_side_by_side_padded(self):
@@ -159,14 +259,14 @@ class DiffMarkupTest(unittest.TestCase):
         self.assertEqual(out[0], '\x1b[36mheader\n\x1b[0m')
         self.assertEqual(out[1], '\x1b[33m--- old\n\x1b[0m')
         self.assertEqual(out[2], '\x1b[33m+++ new\n\x1b[0m')
-        self.assertEqual(out[3], '\x1b[1;36mhunk header\n\x1b[0m')
-        self.assertEqual(out[4], '\x1b[1;34m@@ -1,5 +1,5 @@\n\x1b[0m')
+        self.assertEqual(out[3], '\x1b[36mhunk header\n\x1b[0m')
+        self.assertEqual(out[4], '\x1b[34m@@ -1,5 +1,5 @@\n\x1b[0m')
         self.assertEqual(
             out[5],
             '\x1b[33m1\x1b[0m '
-            '\x1b[31m\x1b[7m\x1b[31mh\x1b[0m\x1b[31mhello\x1b[0m  '
+            '\x1b[31m\x1b[7m\x1b[31m_\x1b[0m\x1b[31mhello\x1b[0m  '
             '\x1b[0m\x1b[33m1\x1b[0m '
-            '\x1b[32mhello\x1b[7m\x1b[32mo\x1b[0m\n')
+            '\x1b[32mhello\x1b[7m\x1b[32m+\x1b[0m\x1b[32m\x1b[0m\n')
         self.assertEqual(
             out[6],
             '\x1b[33m '
@@ -182,21 +282,21 @@ class DiffMarkupTest(unittest.TestCase):
         self.assertEqual(
             out[8],
             '\x1b[33m3\x1b[0m '
-            '\x1b[1;31mgarb\x1b[0m '
+            '\x1b[31mgarb\x1b[0m '
             '\x1b[0m\x1b[33m '
             '\x1b[0m \n')
         self.assertEqual(
             out[9],
             '\x1b[33m4\x1b[0m '
-            '\x1b[31m\x1b[4m\x1b[31mA\x1b[0m\x1b[31mgain\x1b[0m   '
+            '\x1b[31m\x1b[7m\x1b[31mAgain\x1b[0m\x1b[31m\x1b[0m   '
             '\x1b[0m\x1b[33m4\x1b[0m '
-            '\x1b[32m\x1b[4m\x1b[32ma\x1b[0m\x1b[32mgain\x1b[0m\n')
+            '\x1b[32m\x1b[7m\x1b[32magain\x1b[0m\x1b[32m\x1b[0m\n')
         self.assertEqual(
             out[10],
             '\x1b[33m5\x1b[0m '
-            '\x1b[31m\x1b[7m\x1b[31m      \x1b[0m\x1b[1;35m>\x1b[0m '
+            '\x1b[31m \x1b[7m\x1b[31m     \x1b[0m\x1b[95m>\x1b[0m '
             '\x1b[0m\x1b[33m5\x1b[0m '
-            '\x1b[32m\x1b[7m\x1b[32m spaced\x1b[0m\n')
+            '\x1b[32m \x1b[7m\x1b[32mspaced\x1b[0m\x1b[32m\x1b[0m\n')
 
     # This test is not valid anymore
     def __test_markup_side_by_side_neg_width(self):
@@ -208,8 +308,8 @@ class DiffMarkupTest(unittest.TestCase):
         self.assertEqual(out[0], '\x1b[36mheader\n\x1b[0m')
         self.assertEqual(out[1], '\x1b[33m--- old\n\x1b[0m')
         self.assertEqual(out[2], '\x1b[33m+++ new\n\x1b[0m')
-        self.assertEqual(out[3], '\x1b[1;36mhunk header\n\x1b[0m')
-        self.assertEqual(out[4], '\x1b[1;34m@@ -1,4 +1,4 @@\n\x1b[0m')
+        self.assertEqual(out[3], '\x1b[36mhunk header\n\x1b[0m')
+        self.assertEqual(out[4], '\x1b[34m@@ -1,4 +1,4 @@\n\x1b[0m')
         self.assertEqual(
             out[5],
             '\x1b[33m1\x1b[0m '
@@ -232,16 +332,16 @@ class DiffMarkupTest(unittest.TestCase):
         self.assertEqual(
             out[8],
             '\x1b[33m3\x1b[0m '
-            '\x1b[1;31mgarb\x1b[0m '
+            '\x1b[31mgarb\x1b[0m '
             '\x1b[0m\x1b[33m '
             '\x1b[0m \n')
         self.assertEqual(
             out[9],
             '\x1b[33m4\x1b[0m '
-            '\x1b[31m\x1b[4m\x1b[31mA\x1b[0m\x1b[31mgain\x1b[0m ' +
+            '\x1b[31m\x1b[7m\x1b[31mAgain\x1b[0m ' +
             (' ' * 75) +
             '\x1b[0m\x1b[33m4\x1b[0m '
-            '\x1b[32m\x1b[4m\x1b[32ma\x1b[0m\x1b[32mgain\x1b[0m\n')
+            '\x1b[32m\x1b[7m\x1b[32magain\x1b[0m\n')
 
     def test_markup_side_by_side_off_by_one(self):
         diff = self._init_diff()
@@ -256,14 +356,14 @@ class DiffMarkupTest(unittest.TestCase):
         self.assertEqual(out[0], '\x1b[36mheader\n\x1b[0m')
         self.assertEqual(out[1], '\x1b[33m--- old\n\x1b[0m')
         self.assertEqual(out[2], '\x1b[33m+++ new\n\x1b[0m')
-        self.assertEqual(out[3], '\x1b[1;36mhunk header\n\x1b[0m')
-        self.assertEqual(out[4], '\x1b[1;34m@@ -1,5 +1,5 @@\n\x1b[0m')
+        self.assertEqual(out[3], '\x1b[36mhunk header\n\x1b[0m')
+        self.assertEqual(out[4], '\x1b[34m@@ -1,5 +1,5 @@\n\x1b[0m')
         self.assertEqual(
             out[5],
             '\x1b[33m1\x1b[0m '
-            '\x1b[31m\x1b[7m\x1b[31mh\x1b[0m\x1b[31mhello\x1b[0m '
+            '\x1b[31m\x1b[7m\x1b[31m_\x1b[0m\x1b[31mhello\x1b[0m '
             '\x1b[0m\x1b[33m1\x1b[0m '
-            '\x1b[32mhello\x1b[7m\x1b[32mo\x1b[0m\n')
+            '\x1b[32mhello\x1b[7m\x1b[32m+\x1b[0m\x1b[32m\x1b[0m\n')
         self.assertEqual(
             out[6],
             '\x1b[33m \x1b[0m        '
@@ -278,21 +378,21 @@ class DiffMarkupTest(unittest.TestCase):
         self.assertEqual(
             out[8],
             '\x1b[33m3\x1b[0m '
-            '\x1b[1;31mgarb\x1b[0m '
+            '\x1b[31mgarb\x1b[0m '
             '\x1b[0m\x1b[33m '
             '\x1b[0m \n')
         self.assertEqual(
             out[9],
             '\x1b[33m4\x1b[0m '
-            '\x1b[31m\x1b[4m\x1b[31mA\x1b[0m\x1b[31mgain\x1b[0m  '
+            '\x1b[31m\x1b[7m\x1b[31mAgain\x1b[0m\x1b[31m\x1b[0m  '
             '\x1b[0m\x1b[33m4\x1b[0m '
-            '\x1b[32m\x1b[4m\x1b[32ma\x1b[0m\x1b[32mgain\x1b[0m\n')
+            '\x1b[32m\x1b[7m\x1b[32magain\x1b[0m\x1b[32m\x1b[0m\n')
         self.assertEqual(
             out[10],
             '\x1b[33m5\x1b[0m '
-            '\x1b[31m\x1b[7m\x1b[31m     \x1b[0m\x1b[1;35m>\x1b[0m '
+            '\x1b[31m \x1b[7m\x1b[31m    \x1b[0m\x1b[95m>\x1b[0m '
             '\x1b[0m\x1b[33m5\x1b[0m '
-            '\x1b[32m\x1b[7m\x1b[32m spac\x1b[0m\x1b[1;35m>\x1b[0m\n')
+            '\x1b[32m \x1b[7m\x1b[32mspac\x1b[0m\x1b[95m>\x1b[0m\n')
 
     def test_markup_side_by_side_wrapped(self):
         diff = self._init_diff()
@@ -307,20 +407,20 @@ class DiffMarkupTest(unittest.TestCase):
         self.assertEqual(out[0], '\x1b[36mheader\n\x1b[0m')
         self.assertEqual(out[1], '\x1b[33m--- old\n\x1b[0m')
         self.assertEqual(out[2], '\x1b[33m+++ new\n\x1b[0m')
-        self.assertEqual(out[3], '\x1b[1;36mhunk header\n\x1b[0m')
-        self.assertEqual(out[4], '\x1b[1;34m@@ -1,5 +1,5 @@\n\x1b[0m')
+        self.assertEqual(out[3], '\x1b[36mhunk header\n\x1b[0m')
+        self.assertEqual(out[4], '\x1b[34m@@ -1,5 +1,5 @@\n\x1b[0m')
         self.assertEqual(
             out[5],
             '\x1b[33m1\x1b[0m '
-            '\x1b[31m\x1b[7m\x1b[31mh\x1b[0m\x1b[31mhel\x1b[0m\x1b[1;35m>\x1b[0m '  # nopep8
+            '\x1b[31m\x1b[7m\x1b[31m_\x1b[0m\x1b[31mhel\x1b[0m\x1b[95m>\x1b[0m '  # nopep8
             '\x1b[0m\x1b[33m1\x1b[0m '
-            '\x1b[32mhell\x1b[0m\x1b[1;35m>\x1b[0m\n')
+            '\x1b[32mhell\x1b[0m\x1b[95m>\x1b[0m\n')
         self.assertEqual(
             out[6],
             '\x1b[33m \x1b[0m       '
             '\x1b[0m\x1b[33m2\x1b[0m '
             ''
-            '\x1b[32mspam\x1b[0m\x1b[1;35m>\x1b[0m\n')
+            '\x1b[32mspam\x1b[0m\x1b[95m>\x1b[0m\n')
         self.assertEqual(
             out[7],
             '\x1b[33m2\x1b[0m '
@@ -330,21 +430,21 @@ class DiffMarkupTest(unittest.TestCase):
         self.assertEqual(
             out[8],
             '\x1b[33m3\x1b[0m '
-            '\x1b[1;31mgarb\x1b[0m '
+            '\x1b[31mgarb\x1b[0m '
             '\x1b[0m\x1b[33m '
             '\x1b[0m \n')
         self.assertEqual(
             out[9],
             '\x1b[33m4\x1b[0m '
-            '\x1b[31m\x1b[4m\x1b[31mA\x1b[0m\x1b[31mgain\x1b[0m '
+            '\x1b[31m\x1b[7m\x1b[31mAgain\x1b[0m\x1b[31m\x1b[0m '
             '\x1b[0m\x1b[33m4\x1b[0m '
-            '\x1b[32m\x1b[4m\x1b[32ma\x1b[0m\x1b[32mgain\x1b[0m\n')
+            '\x1b[32m\x1b[7m\x1b[32magain\x1b[0m\x1b[32m\x1b[0m\n')
         self.assertEqual(
             out[10],
             '\x1b[33m5\x1b[0m '
-            '\x1b[31m\x1b[7m\x1b[31m    \x1b[0m\x1b[1;35m>\x1b[0m '
+            '\x1b[31m \x1b[7m\x1b[31m   \x1b[0m\x1b[95m>\x1b[0m '
             '\x1b[0m\x1b[33m5\x1b[0m '
-            '\x1b[32m\x1b[7m\x1b[32m spa\x1b[0m\x1b[1;35m>\x1b[0m\n')
+            '\x1b[32m \x1b[7m\x1b[32mspa\x1b[0m\x1b[95m>\x1b[0m\n')
 
     def test_markup_side_by_side_tabbed(self):
         diff = self._init_diff()
@@ -359,14 +459,14 @@ class DiffMarkupTest(unittest.TestCase):
         self.assertEqual(out[0], '\x1b[36mheader\n\x1b[0m')
         self.assertEqual(out[1], '\x1b[33m--- old\n\x1b[0m')
         self.assertEqual(out[2], '\x1b[33m+++ new\n\x1b[0m')
-        self.assertEqual(out[3], '\x1b[1;36mhunk header\n\x1b[0m')
-        self.assertEqual(out[4], '\x1b[1;34m@@ -1,5 +1,5 @@\n\x1b[0m')
+        self.assertEqual(out[3], '\x1b[36mhunk header\n\x1b[0m')
+        self.assertEqual(out[4], '\x1b[34m@@ -1,5 +1,5 @@\n\x1b[0m')
         self.assertEqual(
             out[5],
             '\x1b[33m1\x1b[0m '
-            '\x1b[31m\x1b[7m\x1b[31mh\x1b[0m\x1b[31mhello\x1b[0m   '
+            '\x1b[31m\x1b[7m\x1b[31m_\x1b[0m\x1b[31mhello\x1b[0m   '
             '\x1b[0m\x1b[33m1\x1b[0m '
-            '\x1b[32mhello\x1b[7m\x1b[32mo\x1b[0m\n')
+            '\x1b[32mhello\x1b[7m\x1b[32m+\x1b[0m\x1b[32m\x1b[0m\n')
         self.assertEqual(
             out[6],
             '\x1b[33m '
@@ -382,21 +482,21 @@ class DiffMarkupTest(unittest.TestCase):
         self.assertEqual(
             out[8],
             '\x1b[33m3\x1b[0m '
-            '\x1b[1;31mgarb\x1b[0m '
+            '\x1b[31mgarb\x1b[0m '
             '\x1b[0m\x1b[33m '
             '\x1b[0m \n')
         self.assertEqual(
             out[9],
             '\x1b[33m4\x1b[0m '
-            '\x1b[31m\x1b[4m\x1b[31mA\x1b[0m\x1b[31mgain\x1b[0m    '
+            '\x1b[31m\x1b[7m\x1b[31mAgain\x1b[0m\x1b[31m\x1b[0m    '
             '\x1b[0m\x1b[33m4\x1b[0m '
-            '\x1b[32m\x1b[4m\x1b[32ma\x1b[0m\x1b[32mgain\x1b[0m\n')
+            '\x1b[32m\x1b[7m\x1b[32magain\x1b[0m\x1b[32m\x1b[0m\n')
         self.assertEqual(
             out[10],
             '\x1b[33m5\x1b[0m '
-            '\x1b[31m\x1b[7m\x1b[31m  tabbed\x1b[0m '
+            '\x1b[31m \x1b[7m\x1b[31m tabbed\x1b[0m\x1b[31m\x1b[0m '
             '\x1b[0m\x1b[33m5\x1b[0m '
-            '\x1b[32m\x1b[7m\x1b[32m spaced\x1b[0m\n')
+            '\x1b[32m \x1b[7m\x1b[32mspaced\x1b[0m\x1b[32m\x1b[0m\n')
 
 
 class UnifiedDiffTest(unittest.TestCase):
@@ -456,7 +556,7 @@ class UnifiedDiffTest(unittest.TestCase):
 class DiffParserTest(unittest.TestCase):
 
     def test_parse_invalid_hunk_meta(self):
-        patch = """\
+        patch = b"""\
 spam
 --- a
 +++ b
@@ -466,10 +566,10 @@ spam
         items = patch.splitlines(True)
         stream = iter(items)
         parser = ydiff.DiffParser(stream)
-        self.assertRaises(RuntimeError, list, parser.get_diff_generator())
+        self.assertRaises(RuntimeError, list, parser.parse())
 
     def test_parse_dangling_header(self):
-        patch = """\
+        patch = b"""\
 --- a
 +++ b
 @@ -1,2 +1,2 @@
@@ -482,7 +582,7 @@ spam
         stream = iter(items)
         parser = ydiff.DiffParser(stream)
 
-        out = list(parser.get_diff_generator())
+        out = list(parser.parse())
         self.assertEqual(len(out), 2)
         self.assertEqual(len(out[1]._headers), 1)
         self.assertEqual(out[1]._headers[0], 'spam\n')
@@ -491,7 +591,7 @@ spam
         self.assertEqual(len(out[1]._hunks), 0)
 
     def test_parse_missing_new_path(self):
-        patch = """\
+        patch = b"""\
 --- a
 +++ b
 @@ -1,2 +1,2 @@
@@ -503,10 +603,10 @@ spam
         items = patch.splitlines(True)
         stream = iter(items)
         parser = ydiff.DiffParser(stream)
-        self.assertRaises(AssertionError, list, parser.get_diff_generator())
+        self.assertRaises(AssertionError, list, parser.parse())
 
     def test_parse_missing_hunk_meta(self):
-        patch = """\
+        patch = b"""\
 --- a
 +++ b
 @@ -1,2 +1,2 @@
@@ -520,7 +620,7 @@ spam
         stream = iter(items)
         parser = ydiff.DiffParser(stream)
 
-        out = list(parser.get_diff_generator())
+        out = list(parser.parse())
         self.assertEqual(len(out), 2)
         self.assertEqual(len(out[1]._headers), 0)
         self.assertEqual(out[1]._old_path, '--- c\n')
@@ -528,7 +628,7 @@ spam
         self.assertEqual(len(out[1]._hunks), 0)
 
     def test_parse_missing_hunk_list(self):
-        patch = """\
+        patch = b"""\
 --- a
 +++ b
 @@ -1,2 +1,2 @@
@@ -542,10 +642,10 @@ spam
         items = patch.splitlines(True)
         stream = iter(items)
         parser = ydiff.DiffParser(stream)
-        self.assertRaises(AssertionError, list, parser.get_diff_generator())
+        self.assertRaises(AssertionError, list, parser.parse())
 
     def test_parse_only_in_dir(self):
-        patch = """\
+        patch = b"""\
 --- a
 +++ b
 @@ -1,2 +1,2 @@
@@ -564,7 +664,7 @@ Only in foo: foo
         stream = iter(items)
         parser = ydiff.DiffParser(stream)
 
-        out = list(parser.get_diff_generator())
+        out = list(parser.parse())
         self.assertEqual(len(out), 3)
         self.assertEqual(len(out[1]._hunks), 0)
         self.assertEqual(out[1]._headers, ['Only in foo: foo\n'])
@@ -572,7 +672,7 @@ Only in foo: foo
         self.assertEqual(len(out[2]._hunks[0]._hunk_list), 3)
 
     def test_parse_only_in_dir_at_last(self):
-        patch = """\
+        patch = b"""\
 --- a
 +++ b
 @@ -1,2 +1,2 @@
@@ -585,13 +685,13 @@ Only in foo: foo
         stream = iter(items)
         parser = ydiff.DiffParser(stream)
 
-        out = list(parser.get_diff_generator())
+        out = list(parser.parse())
         self.assertEqual(len(out), 2)
         self.assertEqual(len(out[1]._hunks), 0)
         self.assertEqual(out[1]._headers, ['Only in foo: foo\n'])
 
     def test_parse_binary_differ_diff_ru(self):
-        patch = """\
+        patch = b"""\
 --- a
 +++ b
 @@ -1,2 +1,2 @@
@@ -610,7 +710,7 @@ Binary files a/1.pdf and b/1.pdf differ
         stream = iter(items)
         parser = ydiff.DiffParser(stream)
 
-        out = list(parser.get_diff_generator())
+        out = list(parser.parse())
         self.assertEqual(len(out), 3)
         self.assertEqual(len(out[1]._hunks), 0)
         self.assertEqual(out[1]._old_path, '')
@@ -621,7 +721,7 @@ Binary files a/1.pdf and b/1.pdf differ
         self.assertEqual(len(out[2]._hunks[0]._hunk_list), 3)
 
     def test_parse_binary_differ_git(self):
-        patch = """\
+        patch = b"""\
 diff --git a/foo b/foo
 index 529d8a3..ad71911 100755
 --- a/foo
@@ -646,7 +746,7 @@ index 529e8a3..ad71921 100755
         stream = iter(items)
         parser = ydiff.DiffParser(stream)
 
-        out = list(parser.get_diff_generator())
+        out = list(parser.parse())
         self.assertEqual(len(out), 3)
         self.assertEqual(len(out[1]._hunks), 0)
         self.assertEqual(out[1]._old_path, '')
@@ -657,7 +757,7 @@ index 529e8a3..ad71921 100755
         self.assertEqual(len(out[2]._hunks[0]._hunk_list), 3)
 
     def test_parse_svn_prop(self):
-        patch = """\
+        patch = b"""\
 --- a
 +++ b
 Added: svn:executable
@@ -671,7 +771,7 @@ Added: svn:keywords
         items = patch.splitlines(True)
         stream = iter(items)
         parser = ydiff.DiffParser(stream)
-        out = list(parser.get_diff_generator())
+        out = list(parser.parse())
         self.assertEqual(len(out), 1)
         self.assertEqual(len(out[0]._hunks), 2)
 
@@ -687,7 +787,7 @@ class MainTest(unittest.TestCase):
         self._cwd = os.getcwd()
         self._ws = tempfile.mkdtemp(prefix='test_ydiff')
         self._non_ws = tempfile.mkdtemp(prefix='test_ydiff')
-        cmd = ('cd %s; git init; git config user.name me; '
+        cmd = ('set -o errexit; cd %s; git init; git config user.name me; '
                'git config user.email me@example.org') % self._ws
         subprocess.call(cmd, shell=True, stdout=subprocess.PIPE)
         self._change_file('init')
@@ -698,18 +798,18 @@ class MainTest(unittest.TestCase):
         subprocess.call(cmd)
 
     def _change_file(self, text):
-        cmd = ['/bin/sh', '-c',
+        cmd = ['/bin/sh', '-ec',
                'cd %s; echo "%s" > foo' % (self._ws, text)]
         subprocess.call(cmd)
 
     def _commit_file(self):
-        cmd = ['/bin/sh', '-c',
+        cmd = ['/bin/sh', '-ec',
                'cd %s; git add foo; git commit foo -m update' % self._ws]
         subprocess.call(cmd, stdout=subprocess.PIPE)
 
     def test_preset_options(self):
         os.environ['YDIFF_OPTIONS'] = '--help'
-        self.assertRaises(SystemExit, ydiff.main)
+        self.assertRaises(SystemExit, ydiff._main)
         os.environ.pop('YDIFF_OPTIONS', None)
 
     def test_read_diff(self):
@@ -717,7 +817,7 @@ class MainTest(unittest.TestCase):
         self._change_file('read_diff')
 
         os.chdir(self._ws)
-        ret = ydiff.main()
+        ret = ydiff._main()
         os.chdir(self._cwd)
         self.assertEqual(ret, 0)
 
@@ -729,21 +829,21 @@ class MainTest(unittest.TestCase):
         self._commit_file()
 
         os.chdir(self._ws)
-        ret = ydiff.main()
+        ret = ydiff._main()
         os.chdir(self._cwd)
         self.assertEqual(ret, 0)
 
     def _test_read_diff_neg(self):
         sys.argv = sys.argv[:1]
         os.chdir(self._non_ws)
-        ret = ydiff.main()
+        ret = ydiff._main()
         os.chdir(self._cwd)
         self.assertNotEqual(ret, 0)
 
     def _test_read_log_neg(self):
         sys.argv = [sys.argv[0], '--log']
         os.chdir(self._non_ws)
-        ret = ydiff.main()
+        ret = ydiff._main()
         os.chdir(self._cwd)
         self.assertNotEqual(ret, 0)
 
